# ADR-0004: File Partitioning and Chunking Strategy

## Status

Proposed

## Context

Efficient file storage and transfer are critical to our system, particularly because files must be securely encrypted on the client before transmission. To meet our requirements, files are partitioned into discrete chunks to enable efficient deduplication, versioning, and reassembly (FR-2). This strategy also supports deterministic simulation testing of disk latency, failures, and other adverse conditions, thereby ensuring robust performance and stability under both normal and simulated failure conditions (NFR-2, NFR-3, NFR-6).

## Decision

We will adopt a fixed-size chunking strategy for processing files:

- **Fixed-Size Chunking:**
  Files will be divided into fixed-size chunks (e.g., 1 MB per chunk). This method simplifies chunk management by providing predictable boundaries for deduplication, encryption, and reassembly.

  - **Advantages:**
    - Simplicity and predictability in processing.
    - Straightforward reassembly of files.
    - Efficient deduplication based on consistent chunk sizes.

- **Deterministic Simulation Support:**
  The fixed-size chunking mechanism will be designed to integrate seamlessly with our deterministic simulation framework. This ensures that we can simulate disk latency, disk failures, and other adverse storage conditions during both the partitioning and reassembly processes (NFR-2, NFR-3, NFR-6).

- **Data Integrity:**
  Each chunk will be individually encrypted and indexed with a cryptographic hash (e.g., SHA-256) to guarantee data integrity during storage and reassembly, supporting our goal of secure, reliable storage.

## Consequences

- **Advantages:**

  - **Efficiency and Simplicity:** Fixed-size chunks simplify the overall design, making it easier to manage and duplicate data, thereby enhancing system performance.
  - **Robust Testing:** The defined chunking approach easily integrates with our deterministic simulation framework, enabling rigorous testing under simulated adverse conditions.
  - **Predictable Behavior:** Known chunk sizes facilitate efficient planning for both storage and network resource allocation.

- **Trade-offs:**
  - **Chunk Boundary Issues:** Fixed-size chunking may lead to some inefficiency when file updates occur at arbitrary boundaries, resulting in reprocessing of entire chunks even if only a small portion is modified.
  - **Lack of Adaptive Chunking:** Without content-defined chunking, there is less optimization for files that have variable update patterns. However, for our use case, the simplicity and performance benefits of fixed-size chunking outweigh this limitation.

## References to Requirements

- **Functional Requirements:**

  - FR-2: File Partitioning & Chunk Management – mandates that files be split into chunks to support deduplication and efficient updating.

- **Non-Functional Requirements:**
  - NFR-2: Performance & Responsiveness – the chunking process must maintain performance even under simulated disk and network latency conditions.
  - NFR-3: Scalability & Capacity – efficient chunking supports overall system scalability by minimizing redundant data storage.
  - NFR-6: Deployability & Maintainability – a simpler chunking strategy eases deployment and testing, including deterministic simulations of adverse conditions.

## Conclusion

The decision to use a fixed-size chunking strategy meets our system's need for efficient file partitioning that supports secure deduplication, versioning, and reliable reassembly. By limiting our approach to fixed-size chunking, we simplify the design and implementation, ensuring that our system remains robust and easy to test under a variety of conditions. This decision directly supports our functional and non-functional requirements and reinforces our commitment to building a secure, efficient, and maintainable file storage solution.
