# ADR-0005: Automatic Replication and Redundancy Strategy

## Status

Proposed

## Context

Our file storage system must ensure high data availability and fault tolerance regardless of node, network, or disk failures. According to our functional requirements (FR-5), file chunks need to be replicated across nodes to maintain redundancy, and our non-functional requirements (NFR-2, NFR-3, NFR-6) demand robust behavior even under simulated adverse conditions such as network and disk failures. In order to keep the user experience simple and focused on privacy and control, the replication process should be fully automatic and transparent with no need for direct user configuration.

## Decision

We will implement an automatic replication strategy that is integrated within the system's internal management layers, making replication entirely transparent to the user and client application. The key decisions are as follows:

- **Automatic Replication Management:**
  The system will autonomously monitor node availability and data integrity. When a node failure or data inconsistency is detected, the system automatically replicates the affected file chunks to other nodes to maintain the desired redundancy level. This process is initiated by an internal monitoring mechanism and does not require any user intervention.

- **Replication Under Deterministic Simulation:**
  The replication operations will integrate with our deterministic simulation framework. This allows controlled testing of scenarios such as network latency and disk failures, ensuring that the automatic replication mechanism reliably maintains data availability under a variety of conditions (NFR-2, NFR-3, NFR-6).

- **Data Integrity Verification:**
  Each file chunk is associated with a cryptographic hash (e.g., SHA-256) to verify its integrity before and after replication. The verification process is automatically performed to ensure that replicated data remains consistent and unaltered in all nodes, contributing to the overall auditability and transparency of the system (FR-5, NFR-5).

- **Transparent Operation:**
  All replication processes are handled internally by the system. No replication policy or replication-related configuration is exposed to the user. This design choice reinforces our mission to provide a secure and user-centric solution where complex technical details—such as data redundancy strategies—are completely abstracted from the end user.

## Consequences

- **Advantages:**

  - **Simplicity for Users:** The replication mechanism is entirely automatic, ensuring a frictionless user experience where data availability and redundancy are maintained without any manual configuration.
  - **Enhanced Robustness:** The automatic system continuously monitors and resolves data integrity issues, ensuring minimal downtime and high availability of file data even under network and disk failure conditions.
  - **Testability:** Integration with the deterministic simulation framework enables rigorous testing of replication processes under varied adverse conditions, ensuring reliable performance (NFR-2, NFR-3, NFR-6).

- **Trade-offs:**
  - **System Complexity:** Automating replication and integrating continuous monitoring require additional internal system complexity. However, this complexity is encapsulated so that users remain unaffected.
  - **Resource Overhead:** While automatic replication may use additional resources for data duplication and monitoring, this overhead is justified by the benefits of improved resilience and data availability.
  - **Development Effort:** Developing a robust, automatic replication mechanism demands a comprehensive design and extensive testing, but it aligns with our overall goals of fault tolerance and user-centric design.

## References to Requirements

- **Functional Requirements:**

  - FR-5: Replication and Redundancy Management – Ensures file chunks are replicated across nodes for continuous data availability.

- **Non-Functional Requirements:**
  - NFR-2: Performance & Responsiveness – Replication must operate efficiently even under simulated adverse network and disk conditions.
  - NFR-3: Scalability & Capacity – Automatic replication supports a growing number of files and nodes while sustaining performance.
  - NFR-6: Deployability & Maintainability – The replication process, managed automatically by the system, should simplify deployment and support rigorous deterministic simulation testing.

## Conclusion

The chosen automatic replication and redundancy strategy meets our system’s needs by ensuring continuous data availability and integrity through an entirely transparent process. By automating replication management, integrating deterministic simulation testing, and verifying data integrity through cryptographic measures, the strategy fulfills our functional and non-functional requirements. This decision supports our mission by maintaining a resilient, secure storage environment while keeping the complexity hidden from the end user.
