# ADR-0012: Storage Backends and Data Persistence Strategy

## Status

Proposed

## Context

Our self-hosted file storage system needs to ensure that encrypted file chunks are reliably stored and remain available even in the event of hardware, disk, or network failures. To meet this challenge, our system must use a storage backend and data persistence strategy that supports both local storage and distributed redundancy. This is essential for:

- **File Management and Deduplication (FR-2):** Ensuring that file chunks are correctly managed and can be efficiently retrieved using content addressable methods (SHA-256 indexing).
- **Replication and Redundancy (FR-5):** Guaranteeing that data remains available across nodes even when individual nodes fail.

Furthermore, our strategy must align with non-functional requirements such as:

- **Security & Privacy (NFR-1):** Data must be encrypted and integrity-verified to prevent unauthorized modifications.
- **Performance & Responsiveness (NFR-2):** The system ought to remain fast and efficient, even when simulating adverse disk or network conditions.
- **Scalability (NFR-3):** The storage strategy must support system growth in a modular fashion.
- **Deployability & Maintainability (NFR-6):** The design should facilitate deterministic testing and easy maintenance.

## Decision

We will implement a hybrid storage backend strategy that includes the following elements:

- **Local Filesystem Storage:**
  Each node will manage its file storage on a local filesystem. Key features include:

  - **Write-Ahead Logging:** Operations will be logged prior to execution to support crash recovery.
  - **Content Addressable Storage:** File chunks are indexed using cryptographic hashes (e.g., SHA-256) to ensure integrity, prevent duplication, and streamline retrieval.

- **Distributed Redundancy Through Replication:**

  - **Automatic Replication:** As outlined in ADR-0005, file chunks will be replicated to multiple nodes automatically. This replication ensures that data is available even if one or more nodes are offline.
  - **Seamless Recovery:** In the event of a node failure, redundant copies on other nodes will be used to restore data, thus maintaining high availability.

- **Data Integrity Verification:**
  Each file chunk is associated with a cryptographic hash to verify its integrity regularly. This continuous verification helps detect data corruption or discrepancies, ensuring that the system fulfills its security (NFR-1) and reliability (FR-5) requirements.

- **Deterministic Simulation Support:**
  The storage backend will integrate with our deterministic simulation testing framework. This integration allows us to simulate disk latency, disk failures, and other adverse conditions, ensuring that the system meets the performance (NFR-2) and maintainability (NFR-6) targets under varied scenarios.

## Consequences

- **Advantages:**

  - **High Availability and Resilience:** Distributed replication ensures that data remains accessible despite individual node failures.
  - **Strong Data Integrity:** The use of content addressable storage with SHA-256 indexing and periodic integrity verification maintains data correctness.
  - **Scalability:** Combining local storage with distributed redundancy allows the system to efficiently handle increased loads as the network of nodes grows.
  - **Robust Testing:** Integration with deterministic simulation provides a structured environment to test and validate data persistence and recovery under adverse conditions.

- **Trade-offs:**
  - **System Complexity:** Managing both local storage and distributed replication adds architectural complexity, though this is mitigated by clear logging and automated recovery processes.
  - **Resource Overhead:** Redundant storage increases disk space and network utilization, a necessary trade-off for improved resilience and availability.
  - **Maintenance Effort:** Automated integrity checks and replication processes require ongoing maintenance and monitoring but are essential for long-term reliability.

## References to Requirements

- **Functional Requirements:**

  - **FR-2:** File Partitioning & Chunk Management – Local storage and content addressable indexing support efficient file operations.
  - **FR-5:** Replication and Redundancy Management – Distributed replication ensures that data remains available across nodes.

- **Non-Functional Requirements:**
  - **NFR-1:** Security & Privacy – Encryption and hash-based integrity checks safeguard user data.
  - **NFR-2:** Performance & Responsiveness – The design supports efficient data access even under simulated adverse conditions.
  - **NFR-3:** Scalability & Capacity – The strategy is built to accommodate growth in data volume and the number of nodes.
  - **NFR-6:** Deployability & Maintainability – Integration with deterministic simulation ensures predictable behavior and easier maintenance.

## Conclusion

The proposed storage backend and data persistence strategy, which combines local filesystem storage with distributed replication and robust integrity verification, meets the system’s needs for high availability, security, and scalability. By indexing file chunks using cryptographic hashes and ensuring fault tolerance through automatic replication, this approach aligns with our functional and non-functional requirements. The integration of deterministic simulation testing further validates performance and recovery under varied conditions, ensuring a resilient and maintainable system.
